@@ -341,20 +341,29 @@ LRMI_init(void)
 	 * an explanation
 	 */
 	if (!map_file((void *)0, 0x502,
-	 PROT_READ | PROT_WRITE | PROT_EXEC,
+	 PROT_READ | PROT_WRITE,
 	 MAP_FIXED | MAP_SHARED, "/dev/mem", 0)) {
 		real_mem_deinit();
 		return 0;
 	}
 
-	if (!map_file((void *)0xa0000, 0x100000 - 0xa0000,
-	 PROT_READ | PROT_WRITE | PROT_EXEC,
+	if (!map_file((void *)0xa0000, 0x20000,
+	 PROT_READ | PROT_WRITE,
 	 MAP_FIXED | MAP_SHARED, "/dev/mem", 0xa0000)) {
 		munmap((void *)0, 0x502);
 		real_mem_deinit();
 		return 0;
 	}
 
+	if (!map_file((void *)0xc0000, 0x40000,
+		PROT_READ | PROT_EXEC,
+		MAP_FIXED | MAP_SHARED, "/dev/mem", 0xc0000)) {
+		munmap((void *)0, 0x502);
+		munmap((void *)0xa0000, 0x20000);
+		real_mem_deinit();
+		return 0;
+	}
+
 	/*
 	 Allocate a stack
 	*/
@@ -363,6 +372,18 @@ LRMI_init(void)
 	context.stack_seg = (unsigned int)m >> 4;
 	context.stack_off = DEFAULT_STACK_SIZE;
 
+#if defined(__linux__)
+	/*
+	 The return to 32-bit code (vm86_ret section) is linked
+	 into the absolute address 0x9000.
+	*/
+	context.ret_seg = 0x0900;
+	context.ret_off = 0x0000;
+
+	asm (".pushsection vm86_ret, \"ax\"\nint %0\n.popsection"
+		: /* no return value */
+		: "i" (RETURN_TO_32_INT));
+#else
 	/*
 	 Allocate the return to 32 bit routine
 	*/
@@ -371,9 +392,10 @@ LRMI_init(void)
 	context.ret_seg = (unsigned int)m >> 4;
 	context.ret_off = (unsigned int)m & 0xf;
 
-	((unsigned char *)m)[0] = 0xcd; 	/* int opcode */
+	((unsigned char *)m)[0] = 0xcd;		/* int opcode */
 	((unsigned char *)m)[1] = RETURN_TO_32_INT;
 
+#endif  /* __linux__ */
 	memset(&context.vm, 0, sizeof(context.vm));
 
 	/*
